from db import dbGetMany, dbGetOne
import datetime
from loguru import logger
from decimal import *
from utils import getParam
from Cheetah.Template import Template


def perform(dbh, params):
    year = getParam(params, 'year', datetime.datetime.today().year)
    startDate = datetime.datetime(year, 1, 1, 0, 0, 0)
    endDate = datetime.datetime(year, 12, 31, 23, 59, 59)
    premises = (1, 2)

    # get overhead sums by object, category and timespan
    overheadSums = dbGetMany(
        dbh,
        {
            "statement": 
            """
                select sum(ae.amount) as sum, 
                    aec.description as category, 
                    p.id as house_id,
                    p.description as house,
                    aec.considerminusarea as considerminusarea
                from account_t a,
                    premise_t p,
                    account_entry_t ae,
                    account_entry_category_t aec
                where p.account = a.id and
                    ae.account = a.id and
                    aec.overhead_relevant = 't' and
                    ae.account_entry_category = aec.id and
                    ae.fiscal_year = %(year)s and
                    p.id in %(premises)s
                group by house_id, house, category, considerminusarea
                union
                select 0 as sum,
                    aec.description as category, 
                    p.id as house_id,
                    p.description as house,
                    aec.considerminusarea as considerminusarea
                from account_t a,
                    premise_t p,
                    account_entry_t ae,
                    account_entry_category_t aec
                where p.account = a.id and
                    ae.account = a.id and
                    aec.overhead_relevant = 't' and
                    aec.id not in (select distinct account_entry_category from account_entry_t) and
                    ae.fiscal_year = %(year)s and
                    p.id in %(premises)s
                group by house_id, house, category, considerminusarea
                union
                select 120 as sum,
                    'Waschmaschine' as category,
                    id as house_id,
                    description as house,
                    false as considerminusarea
                from premise_t
                where id in %(premises)s
                order by house_id, category
            """,
            "params": {
               "year": year,
               "premises": premises
            }
        } 
    )
    # logger.info(f"{overheadSums=}")
    for overheadSum in overheadSums:
        logger.info(f"house: {overheadSum['house']}, considerMinusArea: {overheadSum['considerminusarea']}, category: {overheadSum['category']}, sum: {overheadSum['sum']}")

    #subtotal = {}
    #for premise in premises:
    #    v = [ x['sum'] for x in overheadSums if x['house_id'] == premise ]
    #    logger.info(f"{v=}")
    #    subtotal[premise] = sum(v)
    #    logger.info(f"{subtotal=}")

    # get areas and factors
    totalAreas = {}
    flatAreas = dbGetMany(
        dbh,
        {
            "statement":
            """
                select 
                    p.id as house_id,
                    p.description as house,
                    sum(f.area) as flat_area
                  from
                    premise_t p,
                    flat_t f
                  where
                    f.premise = p.id and
                    p.id in %(premises)s
                  group by house_id
            """,
            "params": {
                "premises": premises
            }
        }
    )

    for area in flatAreas:
        logger.info(f"{area['house']=}, {area['flat_area']=}")
        totalAreas[area['house_id']] = { 'flat_area': area['flat_area'] }

    commercialAreas = dbGetMany(
        dbh,
        {
            "statement":
            """
                select 
                    p.id as house_id,
                    p.description as house,
                    coalesce(sum(c.area), 0) as commercial_area
                  from
                    premise_t p full outer join commercial_premise_t c on c.premise = p.id
                  where
                    p.id in %(premises)s
                  group by house_id
            """,
            "params": {
                "premises": premises
            }
        }
    )

    for area in commercialAreas:
        logger.info(f"{area['house']=}, {area['commercial_area']=}")
        totalAreas[area['house_id']] |= { 'commercial_area': area['commercial_area'] }

    minusAreas = dbGetMany(
        dbh,
        {
            "statement":
            """
                select 
                    p.id as house_id,
                    p.description as house,
                    p.minus_area as minus_area
                  from
                    premise_t p
                  where
                    p.id in %(premises)s
                  group by house_id
            """,
            "params": {
                "premises": premises
            }
        }
    )

    for area in minusAreas:
        logger.info(f"{area['house']=}, {area['minus_area']=}")
        totalAreas[area['house_id']] |= { 'minus_area': area['minus_area'] }

    for premise in premises:
        totalAreas[premise]['other_area'] = totalAreas[premise]['commercial_area'] + totalAreas[premise]['minus_area']
        totalAreas[premise]['total_area'] = totalAreas[premise]['flat_area'] + totalAreas[premise]['other_area']
        totalAreas[premise]['flat_factor'] = totalAreas[premise]['flat_area'] / totalAreas[premise]['total_area']
        totalAreas[premise]['other_factor'] = totalAreas[premise]['other_area'] / totalAreas[premise]['total_area']
        totalAreas[premise]['factor_check'] = totalAreas[premise]['flat_factor'] + totalAreas[premise]['other_factor']
    logger.info(f"{totalAreas=}")


            

    ##### ATTENTION: SCRIPT ABORT TEMPORALY INSERTED HERE #####
    return
    ###########################################################


    # get flat tenants by object and timespan with paid overhead and due overhead
    tenants = dbGetMany(
        dbh,
        {
            "statement":
            """
                select t.id as tenant_id,
                    t.salutation as tenant_salutation,
                    t.firstname as tenant_firstname,
                    t.lastname as tenant_lastname,
                    t.address1 as tenant_address1,
                    t.address2 as tenant_address2,
                    t.address3 as tenant_address3,
                    t.zip as tenant_zip,
                    t.city as tenant_city,
                    f.id as flat_id,
                    f.description as flat,
                    f.area as flat_area,
                    p.id as house_id,
                    p.description as house,
                    ty.startdate as startdate,
                    ty.enddate as enddate,
                    t.account as tenant_account
                from tenant_t t,
                    premise_t p,
                    flat_t f,
                    tenancy_t ty
                where ty.tenant = t.id and
                    ty.flat = f.id and
                    ty.startdate <= %(startDate)s and
                    (ty.enddate >= %(endDate)s or ty.enddate is null) and
                    f.premise = p.id and
                    p.id in %(premises)s
                order by house_id, tenant_id
            """,
           "params": {
               "startDate": startDate, 
               "endDate": endDate,
               "premises": premises
           }
        }
    )

    letters = []
    for tenant in tenants:
        letter = {}

        paidTotal = dbGetOne(
            dbh,
            {
                "statement":
                """
                    SELECT sum(amount) AS sum,
                        count(id) AS cnt
                    FROM account_entry_t
                    WHERE account = %(account)s AND
                        account_entry_category = 2 AND
                        fiscal_year = %(year)s
                """,
                "params": {
                    "account": tenant['tenant_account'],
                    "year": year
                }
            }
        )
        tenant['paid_total'] = paidTotal['sum']

        receivableFee = dbGetOne(
            dbh,
            {
                "statement":
                """
                    SELECT sum(amount) * -1 AS sum ,
                        count(id) AS cnt
                    FROM account_entry_t
                    WHERE account = %(account)s AND
                        account_entry_category = 3 AND
                        fiscal_year = %(year)s
                """,
                "params": {
                    "account": tenant['tenant_account'],
                    "year": year
                }
            }
        )
        tenant['receivable_fee'] = receivableFee['sum']
        tenant['rent_time'] = receivableFee['cnt']
        
        tenant['paid_overhead'] = paidTotal['sum'] - receivableFee['sum']

        letter['tenant'] = tenant
        letter['year'] = year
        letter['flat_area'] = totalAreas[tenant['house_id']]['flat_area']

        letters.append(letter)

        logger.info(f"{letter=}")


    printLetters = getParam(params, 'printLetters', False)
    if printLetters:
        letterTemplate = getParam(params, 'letterTemplate', 'jahresabrechnung.tmpl')
        letterPrefix = getParam(params, 'letterPrefix', 'letter')
        letterSuffix = getParam(params, 'letterSuffix', 'tex')

        for letter in letters:
            logger.debug(f"Processing item: {letter}")
            outputFile = f"{letterPrefix}-{letter['tenant']['tenant_id']}.{letterSuffix}"
            tmpl = Template(file=letterTemplate, searchList=[ letter ])
            logger.debug(tmpl)
            with open(outputFile, 'w') as f:
                f.write(str(tmpl))






